//-----------------------------------------------------------------------------
//    Package Definition
//-----------------------------------------------------------------------------
package sim.test.access.api_1_svw_incr_bs_bs;

//-----------------------------------------------------------------------------
//    Imports
//-----------------------------------------------------------------------------
import sim.access.*;
import sim.toolkit.*;
import javacard.framework.*;

import sim.test.util.*;
import sim.test.access.*;


public class API_1_SVW_INCR_BS_BS_1 extends TestToolkitApplet {

    static SIMView theSimView;

    final static short INCR_BUFFER_LENGTH = (short)4;
    final static short RESP_BUFFER_LENGTH = (short)4;
    final static short DATA_BUFFER_LENGTH = (short)4;
    final static short FCI_BUFFER_LENGTH = (short)8;

    static byte[] incr = new byte[INCR_BUFFER_LENGTH];
    static byte[] resp = new byte[RESP_BUFFER_LENGTH];
    static byte[] data = new byte[DATA_BUFFER_LENGTH];
    static byte[] fci  = new byte[FCI_BUFFER_LENGTH];
    static byte[] nullBuffer = null;

    /**
     * The Constructor registers the application for the Event.
     */
    private API_1_SVW_INCR_BS_BS_1() {
        
        // Get a reference to the SIM View.
        theSimView = SIMSystem.getTheSIMView();
        
    }

    public static void install(byte[] bArray, short bOffset, byte bLength) {
        API_1_SVW_INCR_BS_BS_1 testApp = new API_1_SVW_INCR_BS_BS_1();

        // Register the applet with the JCRE.
        testApp.register(bArray, (short)(bOffset+1), bArray[bOffset]);

        // Initialise the data of the test applet.
        testApp.init();
    }

    /**
     * This method will be used to trigger the tests.
     * The application will be triggered by an Formatted SMS PP download event.
     * @param event 
     */
    public void processToolkit(byte event) throws ToolkitException {
        short incrOffset = 0;
        short respOffset = 0;
        short respLength = 0;
        short recOffset = 0;
        short recNumber = 0;
        byte mode = 0;
		short dataOffset = 0;
		short dataLength = 0;
        short fciOffset = 0;
        short fciLength = 0;

        if (event == EVENT_FORMATTED_SMS_PP_ENV) {

            /*--------------------------------------------------------------
             * TEST CASE 01: No EF selected
             *------------------------------------------------------------*/
            try {
                incrOffset = 0;
                respOffset = 0;
                theSimView.increase(incr, incrOffset, resp, respOffset);
                reportTestFailure((byte)TestConstants.TESTCASE01);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE01, 
                        sve.getReason() == SIMViewException.NO_EF_SELECTED);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE01);
            }

            /*--------------------------------------------------------------
             * TEST CASE 02: Increase Cyclic EF, verify response
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_DF_SIMTEST);
                theSimView.select(TestConstants.FID_EF_CARU);

                //Set both records to 00 00 00
                recNumber = 2;
                mode = SIMView.REC_ACC_MODE_PREVIOUS;
                data[0] = data[1] = data[2] = data[3] = (byte)0x00;
                dataOffset = 0;
                dataLength = 3;
                theSimView.updateRecord(recNumber, mode, recOffset, data, 
                                        dataOffset, dataLength);
                theSimView.updateRecord(recNumber, mode, recOffset, data, 
                                        dataOffset, dataLength);

                incrOffset = 0;
                incr[2] = 1;
                respOffset = 0;
                theSimView.increase(incr, incrOffset, resp, respOffset);
                if ((resp[0] != (byte)0x00) || (resp[1] != (byte)0x00) || 
                    (resp[2] != (byte)0x01) || (resp[3] != (byte)0x00))
                    UserException.throwIt((short)0);

                reportTestSuccess((byte)TestConstants.TESTCASE02);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE02);
            }

            /*--------------------------------------------------------------
             * TEST CASE 03: Increase Cyclic EF, verify file contents
             *------------------------------------------------------------*/
            try {
                incrOffset = 1;
                incr[2] = 0;
                incr[3] = 2;
                respOffset = 1;
                theSimView.increase(incr, incrOffset, resp, respOffset);
                if ((resp[0] != (byte)0x00) || (resp[1] != (byte)0x00) || 
                    (resp[2] != (byte)0x00) || (resp[3] != (byte)0x03))
                    UserException.throwIt((short)0);

                resp[3] = 0;
                recNumber = 0;
                mode = SIMView.REC_ACC_MODE_ABSOLUTE_CURRENT;
                recOffset = 0;
                respOffset = 0;
                respLength = 3;
                theSimView.readRecord(recNumber, mode, recOffset, resp,
                                      respOffset, respLength);
                if ((resp[0] != (byte)0x00) || (resp[1] != (byte)0x00) || 
                    (resp[2] != (byte)0x03))
                    UserException.throwIt((short)0);

                reportTestSuccess((byte)TestConstants.TESTCASE03);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE03);
            }

            /*--------------------------------------------------------------
             * TEST CASE 04: incr is null
             *------------------------------------------------------------*/
            try {
                incrOffset = 0;
                respOffset = 0;
                theSimView.increase(nullBuffer, incrOffset, resp, respOffset);
                reportTestFailure((byte)TestConstants.TESTCASE04);
            } catch (NullPointerException npe) {
                reportTestSuccess((byte)TestConstants.TESTCASE04);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE04);
            }

            /*--------------------------------------------------------------
             * TEST CASE 05: incrOffset < 0
             *------------------------------------------------------------*/
            try {
                incrOffset = (short)-1;
                respOffset = 0;
                theSimView.increase(incr, incrOffset, resp, respOffset);
                reportTestFailure((byte)TestConstants.TESTCASE05);
            } catch (ArrayIndexOutOfBoundsException aioobe) {
                reportTestSuccess((byte)TestConstants.TESTCASE05);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE05);
            }

            /*--------------------------------------------------------------
             * TEST CASE 06: incrOffset + 3 > incr.length
             *------------------------------------------------------------*/
            try {
                incrOffset = 2;
                respOffset = 0;
                theSimView.increase(incr, incrOffset, resp, respOffset);
                reportTestFailure((byte)TestConstants.TESTCASE06);
            } catch (ArrayIndexOutOfBoundsException aioobe) {
                reportTestSuccess((byte)TestConstants.TESTCASE06);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE06);
            }

            /*--------------------------------------------------------------
             * TEST CASE 07: Reach Maximum Value
             *------------------------------------------------------------*/
            try {
                incr[0] = incr[1] = incr[2] = (byte)0xFF;
                incrOffset = 0;
                respOffset = 0;
                theSimView.increase(incr, incrOffset, resp, respOffset);
                reportTestFailure((byte)TestConstants.TESTCASE07);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE07, 
                        sve.getReason() == SIMViewException.MAX_VALUE_REACHED);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE07);
            }

            /*--------------------------------------------------------------
             * TEST CASE 08: resp is null
             *------------------------------------------------------------*/
            try {
                incrOffset = 0;
                respOffset = 0;
                theSimView.increase(incr, incrOffset, nullBuffer, respOffset);
                reportTestFailure((byte)TestConstants.TESTCASE08);
            } catch (NullPointerException npe) {
                reportTestSuccess((byte)TestConstants.TESTCASE08);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE08);
            }

            /*--------------------------------------------------------------
             * TEST CASE 09: respOffset < 0
             *------------------------------------------------------------*/
            try {
                incrOffset = 0;
                respOffset = (short)-1;
                theSimView.increase(incr, incrOffset, resp, respOffset);
                reportTestFailure((byte)TestConstants.TESTCASE09);
            } catch (ArrayIndexOutOfBoundsException aioobe) {
                reportTestSuccess((byte)TestConstants.TESTCASE09);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE09);
            }

            /*--------------------------------------------------------------
             * TEST CASE 10: respOffset + recordLength > resp.length
             *------------------------------------------------------------*/
            try {
                incrOffset = 0;
                respOffset = 2;
                theSimView.increase(incr, incrOffset, resp, respOffset);
                reportTestFailure((byte)TestConstants.TESTCASE10);
            } catch (ArrayIndexOutOfBoundsException aioobe) {
                reportTestSuccess((byte)TestConstants.TESTCASE10);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE10);
            }

            /*--------------------------------------------------------------
             * TEST CASE 11: EF is not Cyclic
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_EF_TARU);
                incrOffset = 0;
                respOffset = 0;
                try {
                    theSimView.increase(incr, incrOffset, resp, respOffset);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.FILE_INCONSISTENT)
                        UserException.throwIt((short)0);
                }

                theSimView.select(TestConstants.FID_EF_LARU);
                incrOffset = 0;
                respOffset = 0;
                try {
                    theSimView.increase(incr, incrOffset, resp, respOffset);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.FILE_INCONSISTENT)
                        UserException.throwIt((short)0);
                }

                reportTestSuccess((byte)TestConstants.TESTCASE11);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE11);
            }

            /*--------------------------------------------------------------
             * TEST CASE 12: Access condition not fulfilled
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_EF_CNIC);
                incrOffset = 0;
                respOffset = 0;
                theSimView.increase(incr, incrOffset, resp, respOffset);
                reportTestFailure((byte)TestConstants.TESTCASE12);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE12, 
                        sve.getReason() == SIMViewException.AC_NOT_FULFILLED);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE12);
            }
        
            /*--------------------------------------------------------------
             * TEST CASE 13: EF is invalidated
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_EF_CARU);

                theSimView.invalidate();

                incrOffset = 0;
                respOffset = 0;

                try {
                    theSimView.increase(incr, incrOffset, resp, respOffset);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.INVALIDATION_STATUS_CONTRADICTION)
                        UserException.throwIt((short)0);
                } finally {
                    theSimView.rehabilitate();
                }

                reportTestSuccess((byte)(byte)TestConstants.TESTCASE13);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE13);
            }

            /*--------------------------------------------------------------
             * TEST CASE 14: Check increase not allowed from FCI
             *------------------------------------------------------------*/
            try {
                fciOffset = 0;
                fciLength = 8;
                theSimView.select(TestConstants.FID_EF_CINA, fci, fciOffset, fciLength);

                // GSM 11.11 (bits&bytes start with 1) - check bit 7 of FCI byte 8 
                // Java (bits&bytes start with 0) - check bit 6 of FCI byte 7 
                if ((fci[7] & 0x40) != 0x00)
                    UserException.throwIt((short)0);

                incrOffset = 0;
                respOffset = 0;

                try {
                    theSimView.increase(incr, incrOffset, resp, respOffset);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.FILE_INCONSISTENT)
                        UserException.throwIt((short)0);
                }

                reportTestSuccess((byte)(byte)TestConstants.TESTCASE14);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE14);
            }

        } // end if
    }
}
