//-----------------------------------------------------------------------------
//    Package Definition
//-----------------------------------------------------------------------------
package sim.test.access.api_1_svw_slcts;

//-----------------------------------------------------------------------------
//    Imports
//-----------------------------------------------------------------------------
import sim.toolkit.*;
import sim.access.*;
import javacard.framework.*;

import sim.test.util.*;
import sim.test.access.*;


public class API_1_SVW_SLCTS_1 extends TestToolkitApplet {

    private SIMView theSIMView;

    final static short RESP_BUFFER_LENGTH = (short)20;
    final static short DATA_BUFFER_LENGTH = (short)20;

    static byte[] data = new byte[DATA_BUFFER_LENGTH];
    static byte[] resp = new byte[RESP_BUFFER_LENGTH];
    static byte[] nullBuffer = null;
  
    /**
     * The Constructor register the application for the Event.
     */
    private API_1_SVW_SLCTS_1() {

        // Get a reference to the SIM View.
        theSIMView = SIMSystem.getTheSIMView();

    }

    public static void install(byte[] bArray, short bOffset, byte bLength) {
        API_1_SVW_SLCTS_1 testApp = new API_1_SVW_SLCTS_1();

        // Register the applet with the JCRE.
        testApp.register(bArray, (short)(bOffset+1), bArray[bOffset]);

        // Initialise the data of the test applet.
        testApp.init();
    }

    /**
     * @param event 
     * In the processToolkit-function the incomming data will be processed. 
     * In accordance to the tags of the TLVs, there will be processed a 
     * GetInkeyCommand or an UPDATECommand. The GetInkeyCommand will be also 
     * used to test the of the ProactiveResponseHandler.
     */
    public void processToolkit (byte event) throws ToolkitException {
        
        short result;

        if (event == EVENT_FORMATTED_SMS_PP_ENV) {

            /*------------------------------------------------------------------
             * TEST CASE 01: Select EF in MF
             *----------------------------------------------------------------*/

            try {
                theSIMView.select(SIMView.FID_EF_ICCID);
                reportTestSuccess((byte)TestConstants.TESTCASE01);

            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE01);
            }
                
            /*------------------------------------------------------------------
             * TEST CASE 02: EF not selected after MF/DF selection
             *----------------------------------------------------------------*/
            try {
                theSIMView.select(SIMView.FID_MF);
                theSIMView.select(SIMView.FID_EF_ICCID);
                theSIMView.select(SIMView.FID_MF);

                result = theSIMView.readBinary((short)0, resp, (short)0, (short)1);

                reportTestFailure((byte)TestConstants.TESTCASE02);

            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE02, 
                        sve.getReason() == SIMViewException.NO_EF_SELECTED);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE02);
            }

            
            /*------------------------------------------------------------------
             * TEST CASE 03: No record is selected after selecting a
             * Linear Fixed EF
             *----------------------------------------------------------------*/
            try {
                theSIMView.select(SIMView.FID_MF);
                theSIMView.select(TestConstants.FID_DF_SIMTEST);
                theSIMView.select(TestConstants.FID_EF_LARU);

                theSIMView.readRecord((short)0, SIMView.REC_ACC_MODE_ABSOLUTE_CURRENT, 
                        (short)0, resp, (short)0, (short)1);

                reportTestFailure((byte)TestConstants.TESTCASE03);

            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE03, 
                        sve.getReason() == SIMViewException.RECORD_NUMBER_NOT_AVAILABLE);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE03);
            }

            /*------------------------------------------------------------------
             * TEST CASE 04: Record pointer in selected cyclic EF
             *----------------------------------------------------------------*/
            try {
                theSIMView.select(SIMView.FID_MF);
                theSIMView.select(TestConstants.FID_DF_SIMTEST);
                theSIMView.select(TestConstants.FID_EF_CARU);

                data[0] = 1; 
                data[1] = 2;
                data[2] = 3;
                theSIMView.updateRecord((short)0, SIMView.REC_ACC_MODE_PREVIOUS, (short)0, data, 
                        (short)0,(short)3);

                theSIMView.select(TestConstants.FID_EF_CARU);
                theSIMView.readRecord((short)0, SIMView.REC_ACC_MODE_ABSOLUTE_CURRENT, (short)0, resp, 
                        (short)0,(short)3);

                if (Util.arrayCompare(data, (short)0, resp, (short)0, (short)3)!=0) {
                    UserException.throwIt((short)0);
                }
 
                reportTestSuccess((byte)TestConstants.TESTCASE04);

            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE04);
            }

            /*------------------------------------------------------------------
               * TEST CASE 05: Selection possibilities
             *----------------------------------------------------------------*/
            try {
                theSIMView.select(SIMView.FID_MF);
                theSIMView.select(SIMView.FID_DF_TELECOM);
                theSIMView.select(SIMView.FID_DF_GRAPHICS);
                theSIMView.select(SIMView.FID_DF_TELECOM);
                theSIMView.select(SIMView.FID_DF_GRAPHICS);
                theSIMView.select(SIMView.FID_MF);
                theSIMView.select(SIMView.FID_DF_GSM);
                theSIMView.select(SIMView.FID_DF_TELECOM);
                theSIMView.select(SIMView.FID_DF_TELECOM);

                reportTestSuccess((byte)TestConstants.TESTCASE05);

            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE05);
            }

            /*------------------------------------------------------------------
             * TEST CASE 06: No selection of "unreachable" file.
             *----------------------------------------------------------------*/
            try {
                theSIMView.select(SIMView.FID_MF);

                theSIMView.select(SIMView.FID_EF_ACM);

                reportTestFailure((byte)TestConstants.TESTCASE06);

            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE06, 
                        sve.getReason() == SIMViewException.FILE_NOT_FOUND);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE06);
            }

        } // end if
    }
}
