//-----------------------------------------------------------------------------
//    Package Definition
//-----------------------------------------------------------------------------
package sim.test.access.api_1_svw_updrsbs_bss;

//-----------------------------------------------------------------------------
//    Imports
//-----------------------------------------------------------------------------
import sim.access.*;
import sim.toolkit.*;
import javacard.framework.*;

import sim.test.util.*;
import sim.test.access.*;


public class API_1_SVW_UPDRSBS_BSS_1 extends TestToolkitApplet {

    static SIMView theSimView;

    final static short RESP_BUFFER_LENGTH = (short)20;
    final static short DATA_BUFFER_LENGTH = (short)20;

    static byte[] data = new byte[DATA_BUFFER_LENGTH];
    static byte[] resp = new byte[RESP_BUFFER_LENGTH];
    static byte[] nullBuffer = null;

    final static byte[] RESP_TESTCASE01 = {
        (byte)0x55, (byte)0x55, (byte)0x55, (byte)0x55, (byte)0x55, 
        (byte)0xFF
    };


    /**
     * The Constructor registers the application for the Event.
     */
    private API_1_SVW_UPDRSBS_BSS_1() {
        // Get a reference to the SIM View.
        theSimView = SIMSystem.getTheSIMView();
    }

    public static void install(byte[] bArray, short bOffset, byte bLength) {
        API_1_SVW_UPDRSBS_BSS_1 testApp = new API_1_SVW_UPDRSBS_BSS_1();

        // Register the applet with the JCRE.
        testApp.register(bArray, (short)(bOffset+1), bArray[bOffset]);

        // Initialise the data of the test applet.
        testApp.init();
    }

    /**
     * This method will be used to trigger the tests.
     * The application will be triggered by an Formatted SMS PP download event.
     * @param event 
     */
    public void processToolkit (byte event) throws ToolkitException {

        short result = 0;
        byte mode = 0;
        short recNumber = 0;
        short recOffset = 0;
        short dataOffset = 0;
        short dataLength = 0;
        short respOffset = 0;
        short respLength = 0;

        if (event == EVENT_FORMATTED_SMS_PP_ENV) {

            /*--------------------------------------------------------------
             * TEST CASE 01: No EF selected
             *------------------------------------------------------------*/
            try {
                recNumber = 1;
                mode = SIMView.REC_ACC_MODE_ABSOLUTE_CURRENT;
                recOffset = 0;
                dataOffset = 0;
                dataLength = 10;
                theSimView.updateRecord(recNumber, mode, recOffset, 
                            data, dataOffset, dataLength);
                reportTestFailure((byte)TestConstants.TESTCASE01);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE01, 
                        sve.getReason() == SIMViewException.NO_EF_SELECTED);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE01);
            }

            /*--------------------------------------------------------------
             * TEST CASE 02: Update Absolute from Linear Fixed EF
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_DF_SIMTEST);
                theSimView.select(TestConstants.FID_EF_LARU);

                recNumber = 2;
                mode = SIMView.REC_ACC_MODE_ABSOLUTE_CURRENT;
                recOffset = 0;
                dataOffset = 0;
                dataLength = 4;
                data[0] = data[1] = data[2] = data[3] = 0x11;
                theSimView.updateRecord(recNumber, mode, recOffset, data, 
                                         dataOffset, dataLength);

                respOffset = 0;
                respLength = 4;
                theSimView.readRecord(recNumber, mode, recOffset, resp,
                                        respOffset, respLength);
                if ((resp[0] != 0x11) || (resp[1] != 0x11) || 
                    (resp[2] != 0x11) || (resp[3] != 0x11)) {
                    UserException.throwIt((short)0);
                }

                reportTestSuccess((byte)TestConstants.TESTCASE02);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE02);
            }
        
            /*--------------------------------------------------------------
             * TEST CASE 03: Update Current from Linear Fixed EF
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_DF_SIMTEST);
                theSimView.select(TestConstants.FID_EF_LARU);

                // Record pointer not set, set it with mode NEXT
                recNumber = 0;
                mode = SIMView.REC_ACC_MODE_NEXT;
                recOffset = 0;
                data[0] = data[1] = data[2] = data[3] = 0x00;
                dataOffset = 0;
                dataLength = 4;
                theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);

                // Write data with mode CURRENT
                recNumber = 0;
                mode = SIMView.REC_ACC_MODE_ABSOLUTE_CURRENT;
                data[0] = data[1] = data[2] = data[3] = 0x22;
                theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);

                theSimView.readRecord(recNumber, mode, recOffset, resp, respOffset, respLength);

                if ((resp[0] != 0x22) || (resp[1] != 0x22) || 
                    (resp[2] != 0x22) || (resp[3] != 0x22)) {
                    UserException.throwIt((short)0);
                }

                reportTestSuccess((byte)TestConstants.TESTCASE03);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE03);
            }
        
            /*--------------------------------------------------------------
             * TEST CASE 04: Update Next from Linear Fixed EF, no record
             * pointer set
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_DF_SIMTEST);
                theSimView.select(TestConstants.FID_EF_LARU);

                recNumber = 0;
                mode = SIMView.REC_ACC_MODE_NEXT;
                recOffset = 0;
                data[0] = data[1] = data[2] = data[3] = 0x33;
                dataOffset = respOffset = 0;
                dataLength = respLength = 4;
                theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);

                mode = SIMView.REC_ACC_MODE_ABSOLUTE_CURRENT;
                theSimView.readRecord(recNumber, mode, recOffset, resp, respOffset, respLength);

                if ((resp[0] != 0x33) || (resp[1] != 0x33) || 
                    (resp[2] != 0x33) || (resp[3] != 0x33)) {
                    UserException.throwIt((short)0);
                }

                reportTestSuccess((byte)TestConstants.TESTCASE04);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE04);
            }

            /*--------------------------------------------------------------
             * TEST CASE 05: Update Next from Linear Fixed EF, record 
             * pointer set
             *------------------------------------------------------------*/
            try {
                recNumber = 0;
                mode = SIMView.REC_ACC_MODE_NEXT;
                recOffset = 0;
                data[0] = data[1] = data[2] = data[3] = 0x44;
                dataOffset = respOffset = 0;
                dataLength = respLength = 4;
                theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);

                mode = SIMView.REC_ACC_MODE_ABSOLUTE_CURRENT;
                theSimView.readRecord(recNumber, mode, recOffset, resp, respOffset, respLength);

                if ((resp[0] != 0x44) || (resp[1] != 0x44) || 
                    (resp[2] != 0x44) || (resp[3] != 0x44)) {
                    UserException.throwIt((short)0);
                }

                reportTestSuccess((byte)TestConstants.TESTCASE05);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE05);
            }

            /*--------------------------------------------------------------
             * TEST CASE 06: Update Next from Linear Fixed EF, no more
             * records
             *------------------------------------------------------------*/
            try {
                recNumber = 0;
                mode = SIMView.REC_ACC_MODE_NEXT;
                recOffset = 0;
                data[0] = data[1] = data[2] = data[3] = 0x55;
                dataOffset = 0;
                dataLength = 4;

                theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);                    	

                reportTestFailure((byte)TestConstants.TESTCASE06);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE06, 
                        sve.getReason() == SIMViewException.RECORD_NUMBER_NOT_AVAILABLE);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE06);
            }
        
            /*--------------------------------------------------------------
             * TEST CASE 07: Update Previous from Linear Fixed EF, no
             * record pointer set
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_DF_SIMTEST);
                theSimView.select(TestConstants.FID_EF_LARU);

                recNumber = 0;
                mode = SIMView.REC_ACC_MODE_PREVIOUS;
                recOffset = 0;
                data[0] = data[1] = data[2] = data[3] = 0x66;
                dataOffset = respOffset = 0;
                dataLength = respLength = 4;
                theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);

                mode = SIMView.REC_ACC_MODE_ABSOLUTE_CURRENT;
                theSimView.readRecord(recNumber, mode, recOffset, resp, respOffset, respLength);

                if ((resp[0] != 0x66) || (resp[1] != 0x66) || 
                    (resp[2] != 0x66) || (resp[3] != 0x66)) {
                    UserException.throwIt((short)0);
                }

                reportTestSuccess((byte)TestConstants.TESTCASE07);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE07);
            }

            /*--------------------------------------------------------------
             * TEST CASE 08: Update Previous from Linear Fixed EF, record 
             * pointer set
             *------------------------------------------------------------*/
            try {
                recNumber = 0;
                mode = SIMView.REC_ACC_MODE_PREVIOUS;
                recOffset = 0;
                data[0] = data[1] = data[2] = data[3] = 0x77;
                dataOffset = respOffset = 0;
                dataLength = respLength = 4;
                theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);

                mode = SIMView.REC_ACC_MODE_ABSOLUTE_CURRENT;
                theSimView.readRecord(recNumber, mode, recOffset, resp, respOffset, respLength);

                if ((resp[0] != 0x77) || (resp[1] != 0x77) || 
                    (resp[2] != 0x77) || (resp[3] != 0x77)) {
                    UserException.throwIt((short)0);
                }

                reportTestSuccess((byte)TestConstants.TESTCASE08);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE08);
            }

            /*--------------------------------------------------------------
             * TEST CASE 09: Update Previous from Linear Fixed EF, no more
             * records
             *------------------------------------------------------------*/
            try {
                recNumber = 0;
                mode = SIMView.REC_ACC_MODE_PREVIOUS;
                recOffset = 0;
                data[0] = data[1] = data[2] = data[3] = (byte)0x88;
                dataOffset = respOffset = 0;
                dataLength = respLength = 4;
                theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);

                reportTestFailure((byte)TestConstants.TESTCASE09);

            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE09, 
                        sve.getReason() == SIMViewException.RECORD_NUMBER_NOT_AVAILABLE);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE09);
            }

            /*--------------------------------------------------------------
             * TEST CASE 10: Update Previous from Cyclic EF
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_DF_SIMTEST);
                theSimView.select(TestConstants.FID_EF_CARU);

                recNumber = 2;
                mode = SIMView.REC_ACC_MODE_ABSOLUTE_CURRENT;
                recOffset = 0;
                respOffset = 0;
                respLength = 3;
                theSimView.readRecord(recNumber, mode, recOffset, resp, respOffset, respLength);

                recNumber = 2;
                mode = SIMView.REC_ACC_MODE_PREVIOUS;
                data[0] = (byte)(resp[0] ^ (byte)0xFF);
                data[1] = (byte)(resp[1] ^ (byte)0xFF);
                data[2] = (byte)(resp[2] ^ (byte)0xFF);
                
                dataOffset = 0;
                dataLength = 3;
                theSimView.updateRecord(recNumber, mode, recOffset, data,  dataOffset, dataLength);	

                recNumber = 0;
                mode = SIMView.REC_ACC_MODE_ABSOLUTE_CURRENT;
                respOffset = 0;
                respLength = 3;
                theSimView.readRecord(recNumber, mode, recOffset, resp, respOffset, respLength);

                if ((resp[0] != data[0]) || (resp[1] != data[1]) || 
                    (resp[2] != data[2])) {
                    UserException.throwIt((short)0);
                }

                reportTestSuccess((byte)TestConstants.TESTCASE10);

            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE10);
            }

            /*--------------------------------------------------------------
             * TEST CASE 11: Update Absolute from Linear Fixed EF 
             * beyond records
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_EF_LARU);

                recNumber = (short)-1;
                mode = SIMView.REC_ACC_MODE_ABSOLUTE_CURRENT;
                recOffset = 0;
                dataOffset = 0;
                dataLength = 4;
                try {
                    theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.RECORD_NUMBER_NOT_AVAILABLE)
                        UserException.throwIt((short)0);
                }

                recNumber = 3;
                try {
                    theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.RECORD_NUMBER_NOT_AVAILABLE)
                        UserException.throwIt((short)0);
                }

                reportTestSuccess((byte)TestConstants.TESTCASE11);

            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE11);
            }

            /*--------------------------------------------------------------
             * TEST CASE 12: No current record in linear fixed EF, 
             * update current
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_EF_LARU);

                // No current record
                recNumber = 0;
                mode = SIMView.REC_ACC_MODE_ABSOLUTE_CURRENT;
                recOffset = 0;
                dataOffset = 0;
                dataLength = 4;
                theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);

                reportTestFailure((byte)TestConstants.TESTCASE12);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE12, 
                        sve.getReason() == SIMViewException.RECORD_NUMBER_NOT_AVAILABLE);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE12);
            }

            /*--------------------------------------------------------------
             * TEST CASE 13: recOffset < 0
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_EF_LARU);

                recNumber = 1;
                mode = SIMView.REC_ACC_MODE_ABSOLUTE_CURRENT;
                recOffset = (short)-1;
                dataOffset = 0;
                dataLength = 4;
                theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);

                reportTestFailure((byte)TestConstants.TESTCASE13);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE13, 
                        sve.getReason() == SIMViewException.OUT_OF_RECORD_BOUNDARIES);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE13);
            }

            /*--------------------------------------------------------------
             * TEST CASE 14: recOffset + dataLength > Record Length 
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_EF_LARU);

                recNumber = 1;
                mode = SIMView.REC_ACC_MODE_ABSOLUTE_CURRENT;
                recOffset = 2;
                dataOffset = 0;
                dataLength = 4;
                theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);

                reportTestFailure((byte)TestConstants.TESTCASE14);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE14, 
                        sve.getReason() == SIMViewException.OUT_OF_RECORD_BOUNDARIES);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE14);
            }

            /*--------------------------------------------------------------
             * TEST CASE 15: Updating with invalid mode 
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_EF_LARU);

                recNumber = 0;
                mode = 1;
                recOffset = 0;
                dataOffset = 0;
                dataLength = 4;
                try {
                    theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.INVALID_MODE)
                        UserException.throwIt((short)0);
                }

                mode = 5;
                try {
                    theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.INVALID_MODE)
                        UserException.throwIt((short)0);
                }

                reportTestSuccess((byte)TestConstants.TESTCASE15);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE15);
            }

            /*--------------------------------------------------------------
             * TEST CASE 16: Updating Cyclic EF with invalid mode
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_DF_SIMTEST);
                theSimView.select(TestConstants.FID_EF_CARU);

                recNumber = 0;
                mode = SIMView.REC_ACC_MODE_NEXT;
                recOffset = 0;
                data[0] = data[1] = data[2] = 0x00;
                dataOffset = 0;
                dataLength = 3;
                try {
                    theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.INVALID_MODE)
                        UserException.throwIt((short)0);
                }

                recNumber = 0;
                mode = SIMView.REC_ACC_MODE_ABSOLUTE_CURRENT;
                try {
                    theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.INVALID_MODE)
                        UserException.throwIt((short)0);
                }

                recNumber = 2;
                mode = SIMView.REC_ACC_MODE_ABSOLUTE_CURRENT;
                try {
                    theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.INVALID_MODE)
                        UserException.throwIt((short)0);
                }

                reportTestSuccess((byte)TestConstants.TESTCASE16);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE16);
            }

            /*--------------------------------------------------------------
             * TEST CASE 17: data is null
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_EF_LARU);

                recNumber = 0;
                recOffset = 0;
                dataOffset = 0;
                dataLength = 4;
                theSimView.updateRecord(recNumber, mode, recOffset, nullBuffer, dataOffset, dataLength);

                reportTestFailure((byte)TestConstants.TESTCASE17);
            } catch (NullPointerException npe) {
                reportTestSuccess((byte)TestConstants.TESTCASE17);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE17);
            }

            /*--------------------------------------------------------------
             * TEST CASE 18: dataOffset < 0
             *-----------------------------------------------------------*/
            try {
                dataOffset = (short)-1;
                dataLength = 10;
                theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);

                reportTestFailure((byte)TestConstants.TESTCASE18);
            } catch (ArrayIndexOutOfBoundsException aioobe) {
                reportTestSuccess((byte)TestConstants.TESTCASE18);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE18);
            }

            /*--------------------------------------------------------------
             * TEST CASE 19: dataLength < 0
             *------------------------------------------------------------*/
            try {
                dataOffset = 0;
                dataLength = (short)-1;
                theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);

                reportTestFailure((byte)TestConstants.TESTCASE19);
            } catch (ArrayIndexOutOfBoundsException aioobe) {
                reportTestSuccess((byte)TestConstants.TESTCASE19);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE19);
            }

            /*--------------------------------------------------------------
             * TEST CASE 20: dataOffset + dataLength > data.length
             *------------------------------------------------------------*/
            try {
                dataOffset = 10;
                dataLength = 11;
                theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);

                reportTestFailure((byte)TestConstants.TESTCASE20);
            } catch (ArrayIndexOutOfBoundsException aioobe) {
                reportTestSuccess((byte)TestConstants.TESTCASE20);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE20);
            }

            /*--------------------------------------------------------------
             * TEST CASE 21: EF is neither Cyclic nor Linear Fixed
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_DF_SIMTEST);
                theSimView.select(TestConstants.FID_EF_TNR);

                dataOffset = 0;
                dataLength = 4;
                    theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);
                reportTestFailure((byte)TestConstants.TESTCASE21);
            } catch (SIMViewException sve) {
                reportTestOutcome((byte)TestConstants.TESTCASE21, 
                        sve.getReason() == SIMViewException.FILE_INCONSISTENT);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE21);
            }

            /*--------------------------------------------------------------
             * TEST CASE 22: Access condition not fulfilled
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_EF_CNU);

                try {
                    mode = SIMView.REC_ACC_MODE_PREVIOUS;
                    recOffset = 0;
                    dataOffset = 0;
                    dataLength = 1;
                    theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.AC_NOT_FULFILLED)
                        UserException.throwIt((short)0);
                }

                theSimView.select(TestConstants.FID_EF_LNU);

                recNumber = 1;
                mode = SIMView.REC_ACC_MODE_ABSOLUTE_CURRENT;
                recOffset = 0;
                dataOffset = 0;
                dataLength = 1;
                try {
                    theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.AC_NOT_FULFILLED)
                        UserException.throwIt((short)0);
                }

                reportTestSuccess((byte)TestConstants.TESTCASE22);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE22);
            }

            /*--------------------------------------------------------------
             * TEST CASE 23: EF is invalidated
             *------------------------------------------------------------*/
            try {
                theSimView.select(TestConstants.FID_EF_CNR);
                mode = SIMView.REC_ACC_MODE_PREVIOUS;

                theSimView.invalidate();

                try {
                    theSimView.updateRecord(recNumber, mode, recOffset, data, dataOffset, dataLength);
                    UserException.throwIt((short)0);
                } catch (SIMViewException sve) {
                    if (sve.getReason() != SIMViewException.INVALIDATION_STATUS_CONTRADICTION)
                        UserException.throwIt((short)0);
                } finally {
                    theSimView.rehabilitate();
                }

                reportTestSuccess((byte)TestConstants.TESTCASE23);
            } catch (Exception e) {
                reportTestFailure((byte)TestConstants.TESTCASE23);
            }

        } // end if
    }
}
