//-----------------------------------------------------------------------------
//    FWK_FWS_INDA_3.java
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//    Package Definition
//-----------------------------------------------------------------------------
package sim.test.framework.fwk_fws_inda;

//-----------------------------------------------------------------------------
//    Imports
//-----------------------------------------------------------------------------
import sim.toolkit.*;
import javacard.framework.*;
import sim.test.util.*;

/**
 * Test Area : sim.toolkit.framework Framework security
 *
 * Applet is triggered on EVENT_FORMATTED_SMS_CB
 *
 * This applet was added for 3GPP TS 51.013 Release 4.
 * It was not present in 3GPP TS 11.13 Release 98.
 *
 * @author 3GPP T3 ad hoc on "Testing API"
 *
 */
public class FWK_FWS_INDA_3 extends TestToolkitApplet
{
	private static final byte NO_TRIGGERING_IN_THIS_TEST_CASE = (byte)0x03;
	private ToolkitRegistry obReg;
	byte testCaseNb = (byte) 0x00;
	byte[] buffer = new byte[25];


    /**
     * Constructor of the applet
     */
    public FWK_FWS_INDA_3() {
		obReg = ToolkitRegistry.getEntry();
		// Triggering on EVENT_FORMATTED_SMS_CB
		obReg.setEvent(EVENT_FORMATTED_SMS_CB);
	}


    /**
     * Method called by the JCRE at the installation of the applet
     */
    public static void install(byte bArray[], short bOffset, byte bLength)
    {
        // Create a new applet instance
        FWK_FWS_INDA_3 thisApplet = new FWK_FWS_INDA_3();

        // Register the new applet instance to the JCRE
        thisApplet.register(bArray, (short)(bOffset+1), bArray[bOffset]);

        // Initialise the data of the test applet
        thisApplet.init();

    }


    /**
       * Method called by the SIM Toolkit Framework
     */
    public void processToolkit(byte event)
    {
		// Result of each test
		boolean bRes = false;

		// Get the system instance of the EnvelopeHandler class
		EnvelopeHandler envHdlr = EnvelopeHandler.getTheHandler();

		// Sets the test case number
		testCaseNb++;

		short length = 0;
		short offset = 0;
		byte data = 0;

		switch ( testCaseNb )
			{
				case (byte) 1 :
				// --------------------------------------------
				// Test Case 4: data ciphered; integrity checked via cryptographic checksum
				//
				// In this test case it is checked that the checksum is verified and the data are deciphered
				// before being passed to the applet
				try {
					bRes = false;
					offset = envHdlr.getSecuredDataOffset();
					length = envHdlr.getSecuredDataLength();
					envHdlr.copyValue(offset, (byte[])buffer, (short)0x00, length);
					data = buffer [0];
				    if ((data == (byte)1) & (length == (byte)1)) {
					    bRes = true;
					}
				}
				catch (Exception e)    {
				    bRes = false;
				}
				reportTestOutcome(testCaseNb, bRes);
				break;

				case (byte) 2 :
				// --------------------------------------------
				// Test Case 5.1: data ciphered; integrity checked via cryptographic checksum
				//
				// In this test case it is checked that applet FWK_FWS_INDA_3 is triggered
				// instead of applet FWK_FWS_INDA_2
				try {
					bRes = false;
					offset = envHdlr.getSecuredDataOffset();
					length = envHdlr.getSecuredDataLength();
					envHdlr.copyValue(offset, (byte[])buffer, (short)0x00, length);
					data = buffer [0];
				    if ((data == (byte)2) & (length == (byte)1)) {
					    bRes = true;
					}
				}
				catch (Exception e)    {
				    bRes = false;
				}
				reportTestOutcome(testCaseNb, bRes);
				reportTestOutcome(NO_TRIGGERING_IN_THIS_TEST_CASE, true);
				break;

				default :
				// --------------------------------------------
				// Test Case 6: data ciphered; integrity checked via cryptographic checksum
				//
				// In this test case it is checked that applet FWK_FWS_INDA_3 is NOT TRIGGERED
				// Therefore, the result is false if this piece of code is executed
				reportTestOutcome(NO_TRIGGERING_IN_THIS_TEST_CASE, false);
				break;

			}

	}
}
