//-----------------------------------------------------------------------------
//    API_2_TKR_IEVSB_1.java
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//    Package Definition
//-----------------------------------------------------------------------------
package sim.test.toolkit.api_2_tkr_ievsb;

//-----------------------------------------------------------------------------
//    Imports
//-----------------------------------------------------------------------------
import sim.toolkit.*;
import javacard.framework.*;
import sim.test.util.*;

/**
 * Test Area : sim.toolkit.ToolkitRegistry.isEventSet(byte event) method
 *
 * Applet is triggered on Install (Install), EVENT_FORMATTED_SMS_PP_ENV
 *
 */
public class API_2_TKR_IEVSB_1 extends TestToolkitApplet {

    // Not allowed events for the setEvent()    method
    private static byte[] NOT_ALLOWED_SET_EVENTS =
                                                {EVENT_MENU_SELECTION,
                                                    EVENT_MENU_SELECTION_HELP_REQUEST,
                                                        EVENT_TIMER_EXPIRATION,
                                                            EVENT_STATUS_COMMAND};

    // Not supported events for the setEvent() method
    private static short NOT_SUPPORTED_SETEVENT_0 = 0;

    private static short SUPPORTED_SETEVENT_INTERVAL_MAX = 24;
    private static short SUPPORTED_SETEVENT_INTERVAL_MIN = -1;

    // Event value range
    private static short MIN_EVENT_NB = -1;
    private static short MAX_EVENT_NB = 24;

    // Events allowed and supported
    private static short MAX_EVENT = 22; //    26 events(-1, 1    to 24, 127) - 4 events not allowed
    private byte [] EventList = new byte[MAX_EVENT];

    private static byte [] MenuEntry =
                        {(byte) 'T',
                        (byte) 'e',
                        (byte) 's',
                        (byte) 't'};

    /**
     * Constructor of the applet
     */
    public API_2_TKR_IEVSB_1() {
    }

    /**
     * Method called by the JCRE at the installation of the applet
     */
    public static void install(byte bArray[], short bOffset, byte bLength) {
        // Create a new applet instance
        API_2_TKR_IEVSB_1 thisApplet = new API_2_TKR_IEVSB_1();

        // Register the new applet instance to the JCRE
        thisApplet.register(bArray, (short)(bOffset+1), (byte) bArray[bOffset]);

        // Initialise the data of the test applet
        thisApplet.init();

        // Creating a menu entry for further use.
        thisApplet.obReg.initMenuEntry(MenuEntry, (short)0, (short)MenuEntry.length,
                                       (byte)0,
                                       false,
                                       (byte)0, (short)0);
    }

     /**
      * Method called by the processToolkit to know if event
      * is an allowed event for the clearEvent() or setEvent() method
      */
    private boolean isEventAllowed(byte event) {
        boolean allowed = true;

        for (short i = ((short) (NOT_ALLOWED_SET_EVENTS.length-1));
            ((i > (short) -1) && (allowed));i--){
            allowed = (NOT_ALLOWED_SET_EVENTS[i] != event);
        }
        return allowed;
    }

     /**
      * Method called by the processToolkit to know if event
      * is a supported event for the setEvent() method
      */
    private boolean isEventSupported(byte event) {
        return  (((event != NOT_SUPPORTED_SETEVENT_0)
                && (event >= SUPPORTED_SETEVENT_INTERVAL_MIN)
                && (event <= SUPPORTED_SETEVENT_INTERVAL_MAX))
                || (event == EVENT_FIRST_COMMAND_AFTER_SELECT));
                        // check if event is not equals to
                        // a not supported event (Interval of values)
    }

    /**
     * Method called by the SIM Toolkit Framework
     */
    public void processToolkit(byte event) {
        // Result of tests
        boolean bRes=true;

        // Number of tests
        byte testCaseNb;

        // Get the system instance of the ProactiveHandler class
        ProactiveHandler proHdlr = ProactiveHandler.getTheHandler() ;

        ProactiveResponseHandler proRespHdlr = null;

        short j=0;

        // Build the allowed and supported events list
        for (short i = MIN_EVENT_NB; i <= MAX_EVENT_NB;i++) {

            if ((this.isEventAllowed((byte) i))
             && (this.isEventSupported((byte) i))) {
                EventList[j] = (byte) i;
                j++;
            }
        }
        EventList[j] = (byte) EVENT_FIRST_COMMAND_AFTER_SELECT;

        // -----------------------------------------------------------------
        // Test Case 1 :Applet calls isEventSet()
        //                for each events ranging from -1 to 24 and 127
        //                except EVENT_FORMATTED_SMS_PP_ENV and
        //                EVENT_MENU_SELECTION.
        //                Shall return false each time
        // -----------------------------------------------------------------

        testCaseNb = (byte)1 ;

        /* all events from -1, 1 to 24 and 127 except 2 and 7 */
        for (short i = 0; (i < (short)(MAX_EVENT-1)) && bRes; i++) {
            if ((EventList[i] != EVENT_FORMATTED_SMS_PP_ENV) && (EventList[i] != EVENT_MENU_SELECTION)) {
                bRes = !obReg.isEventSet((byte) EventList[i]);
            }
        }

        reportTestOutcome(testCaseNb, bRes);

        // -----------------------------------------------------------------
        // Test Case 2 : For EVENT_FORMATTED_SMS_PP_ENV
        //
        //                1 - isEventSet(EVENT_FORMATTED_SMS_PP_ENV)
        //                Shall return true
        // -----------------------------------------------------------------

        testCaseNb = (byte)2;

        bRes = obReg.isEventSet(EVENT_FORMATTED_SMS_PP_ENV);

        reportTestOutcome(testCaseNb, bRes)    ;

        // -----------------------------------------------------------------
        // Test Case 3 : For EVENT_MENU_SELECTION
        //
        //                1 - isEventSet( )
        //                Shall return true
        // -----------------------------------------------------------------

        testCaseNb = (byte)3;

        bRes = obReg.isEventSet(EVENT_MENU_SELECTION);

        reportTestOutcome(testCaseNb, bRes) ;

        // -----------------------------------------------------------------
        // Test Case 4 :
        //                1 - clearEvent(EVENT_FORMATTED_SMS_PP_ENV)
        //                    No exception shall be thrown
        //
        //                2 - isEventSet(EVENT_FORMATTED_SMS_PP_ENV)
        //                    Shall return false
        // -----------------------------------------------------------------

        testCaseNb = (byte)4;
        bRes = false;

        // -----------------------------------------------------------------
        //                1 - clearEvent(EVENT_FORMATTED_SMS_PP_ENV)
        //                    No exception shall be thrown
        // -----------------------------------------------------------------
        try {
            obReg.clearEvent(EVENT_FORMATTED_SMS_PP_ENV);
            bRes = true;
        }
        catch (Exception e) {
            bRes = false;
        }

        // -----------------------------------------------------------------
        //                2 - isEventSet(EVENT_FORMATTED_SMS_PP_ENV)
        //                    Shall return false
        // -----------------------------------------------------------------
        bRes = bRes && !obReg.isEventSet(EVENT_FORMATTED_SMS_PP_ENV);

        reportTestOutcome(testCaseNb, bRes)    ;

        // -----------------------------------------------------------------
        // Test Case 5 :For each SUPPORTED and ALLOWED event for setEvent(),
        //                applet calls:
        //                1 - setEvent() method
        //                    No exception shall be thrown
        //
        //                2 - isEventSet() method
        //                    Shall return true each time
        // -----------------------------------------------------------------

        testCaseNb = (byte)5;
        bRes = true;
        try {

        // -----------------------------------------------------------------
        //    For each SUPPORTED and ALLOWED events, applet calls:
        // -----------------------------------------------------------------
            for (short i = 0; (i < (short)(MAX_EVENT-1)) && bRes;i++) {

                bRes = false;
                if ((this.isEventAllowed((byte) EventList[i]))
                    && (this.isEventSupported((byte) EventList[i]))) {
        // -----------------------------------------------------------------
        //                1 - setEvent() method
        //                    No exception shall be thrown
        // -----------------------------------------------------------------
                    obReg.setEvent((byte) EventList[i]);

        // -----------------------------------------------------------------
        //                2 - isEventSet() method
        //                    Shall return true each time
        // -----------------------------------------------------------------
                    bRes = obReg.isEventSet((byte) EventList[i]);

                    // Clear the setEvent
                    obReg.clearEvent((byte)EventList[i]);
                } else {
                    bRes = true;
                }
            }
        }
        catch (Exception e) {   // No exception should be thrown
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes);

        // -----------------------------------------------------------------
        // Test Case 6 :For each EVENT_MENU_SELECTION_HELP_REQUEST:
        //                1 - isEventSet(EVENT_MENU_SELECTION_HELP_REQUEST)
        //                    shall return false
        //
        //                2 - changeMenuEntry with help supported.
        //                    Shall return true each time
        //
        //                3 - isEventSet(EVENT_MENU_SELECTION_HELP_REQUEST)
        //                    shall return true
        // -----------------------------------------------------------------

        testCaseNb = (byte)6;

        // -----------------------------------------------------------------
        //                1 - isEventSet(EVENT_MENU_SELECTION_HELP_REQUEST)
        //                    shall return false
        // -----------------------------------------------------------------

        bRes = !obReg.isEventSet(EVENT_MENU_SELECTION_HELP_REQUEST);

        // -----------------------------------------------------------------
        //                2 - changeMenuEntry with help supported.
        //                    Shall return true each time
        // -----------------------------------------------------------------

        try {
            obReg.changeMenuEntry((byte) 1,
                                  MenuEntry, (short)0, (short)MenuEntry.length,
                                  (byte)0, true, (byte)0, (short)0);
        }
        catch (Exception e) {
            bRes = false;
        }

        // -----------------------------------------------------------------
        //                3 - isEventSet(EVENT_MENU_SELECTION_HELP_REQUEST)
        //                    shall return true
        // -----------------------------------------------------------------

        bRes = bRes && obReg.isEventSet(EVENT_MENU_SELECTION_HELP_REQUEST);

        reportTestOutcome(testCaseNb, bRes);

        // -----------------------------------------------------------------
        // Test Case 7 : For EVENT_TIMER_EXPIRATION.
        //
        //                1 - isEventSet(EVENT_TIMER_EXPIRATION)
        //                    shall return false
        //
        //                2 - allocateTimer();
        //
        //                3 - isEventSet(EVENT_TIMER_EXPIRATION)
        //                    shall return true
        // -----------------------------------------------------------------

        testCaseNb = (byte)7;

        // -----------------------------------------------------------------
        //                1 - isEventSet(EVENT_TIMER_EXPIRATION)
        //                    shall return false
        // -----------------------------------------------------------------

        bRes = !obReg.isEventSet(EVENT_TIMER_EXPIRATION);

        // -----------------------------------------------------------------
        //                2 - changeMenuEntry with help supported.
        //                    Shall return true each time
        // -----------------------------------------------------------------

        try {
            obReg.allocateTimer();
        }
        catch (Exception e) {
            bRes = false;
        }

        // -----------------------------------------------------------------
        //                3 - isEventSet(EVENT_TIMER_EXPIRATION)
        //                    shall return true
        // -----------------------------------------------------------------

        bRes = bRes && obReg.isEventSet(EVENT_TIMER_EXPIRATION);

        reportTestOutcome(testCaseNb, bRes);

        // -----------------------------------------------------------------
        // Test Case 8 : For Event EVENT_STATUS_COMMAND
        //
        //                1 - isEventSet(EVENT_STATUS_COMMAND)
        //                    shall return false
        //
        //                2 - requestPollInterval(POLL_SYSTEM_DURATION)
        //
        //                3 - isEventSet(EVENT_STATUS_COMMAND)
        //                    shall return true
        // -----------------------------------------------------------------

        testCaseNb = (byte)8;

        // -----------------------------------------------------------------
        //                1 - isEventSet(EVENT_STATUS_COMMAND)
        //                    shall return false
        // -----------------------------------------------------------------

        bRes = !obReg.isEventSet(EVENT_STATUS_COMMAND);

        // -----------------------------------------------------------------
        //                2 - changeMenuEntry with help supported.
        //                    Shall return true each time
        // -----------------------------------------------------------------

        try {
            obReg.requestPollInterval(POLL_SYSTEM_DURATION);
        }
        catch (Exception e) {
            bRes = false;
        }

        // -----------------------------------------------------------------
        //                3 - isEventSet(EVENT_STATUS_COMMAND)
        //                    shall return true
        // -----------------------------------------------------------------

        bRes = bRes && obReg.isEventSet(EVENT_STATUS_COMMAND);

        reportTestOutcome(testCaseNb, bRes);
    }
}
