//-----------------------------------------------------------------------------
//    API_2_ERH_FACRB_BS_1.java
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//    Package Definition
//-----------------------------------------------------------------------------
package sim.test.toolkit.api_2_erh_facrb_bs;

//-----------------------------------------------------------------------------
//    Imports
//-----------------------------------------------------------------------------
import sim.toolkit.* ;
import javacard.framework.* ;
import sim.test.util.* ;

/**
 * Test Area : sim.toolkit.EnvelopeResponseHandler.findAndCompareValue(byte, byte[], short) method
 * 
 * Applet is triggered on Install (Install), EVENT_FORMATTED_SMS_PP_ENV
 *
 * @version 1.0.0 - 01/08/01
 * @author 3GPP T3 SWG API
 *
 * @see sim.toolkit.EnvelopeResponseHandler
 */
public class API_2_ERH_FACRB_BS_1 extends TestToolkitApplet {

    /** Registry entry concerning applet instance */
    private ToolkitRegistry oReg;

    /**
     * Constructor of the applet
     */
    public API_2_ERH_FACRB_BS_1() 
    {
        // Specific registration to Unrecognized Envelope
        obReg.setEvent(EVENT_UNRECOGNIZED_ENVELOPE);
    }


    /**
     * Method called by the JCRE at the installation of the applet
     */
    public static void install(byte bArray[], short bOffset, byte bLength) {
        // Create a new applet instance
        API_2_ERH_FACRB_BS_1 thisApplet = new API_2_ERH_FACRB_BS_1();

        // Register the new applet instance to the JCRE            
        thisApplet.register(bArray, (short)(bOffset+1), bArray[bOffset]);

        // Initialise the data of the test applet
        thisApplet.init();    
    }


    /**
     * Method called by the SIM Toolkit Framework
     */
    public void processToolkit(byte event) {
        // Result of tests
        boolean bRes ;

        // Number of tests
        byte testCaseNb = (byte) 0x00 ;

        // Get the system instance of the EnvelopeResponseHandler class
        EnvelopeResponseHandler EnvRespHdlr = EnvelopeResponseHandler.getTheHandler() ;

        // --------------------------------------------
        // Test Case 1 : Null as compareBuffer
        testCaseNb = (byte) 1 ;
        bRes = false ;

        byte[] TLVBuffer=new byte[20];
        byte[] dstBuffer=new byte[20];

        try {
            //initialize the buffer
            for(short i=1;i<17;i++)
                TLVBuffer[i]=(byte)(i-1);
            TLVBuffer[0]=(byte)4;

            //built the first tlv
            EnvRespHdlr.appendTLV((byte)0x0D,TLVBuffer,(short) 0x00, (short) 16);

            try {
                EnvRespHdlr.findAndCompareValue((byte)0x0D, null, (short)0) ;
            }
            catch (NullPointerException e) {
                bRes = true ;
            }
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 2 : compareOffset >= compareBuffer.length
        testCaseNb = (byte) 2 ;
        bRes = false ;

        try {
            
            try {
                EnvRespHdlr.findAndCompareValue((byte)0x0D, TLVBuffer, (short) 20) ;
                
            } catch (ArrayIndexOutOfBoundsException e) {
                bRes = true ;
            }
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 3 : compareOffset < 0
        testCaseNb = (byte) 3 ;
        bRes = false ;

        try {
            try {
                EnvRespHdlr.findAndCompareValue((byte)0x0D, TLVBuffer, (short)-1) ;
            }
            catch (ArrayIndexOutOfBoundsException e) {
                bRes = true ;
            }
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 4 : compareOffset + length > compareBuffer.length
        testCaseNb = (byte) 4 ; 
        bRes = false ;

        try {
            try {
                EnvRespHdlr.findAndCompareValue((byte)0x0D, TLVBuffer, (short)5) ;
            }
            catch (ArrayIndexOutOfBoundsException e) {
                bRes = true ;
            }
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 5 : length > compareBufferLength
        testCaseNb = (byte) 5 ;
        bRes = false ;


        try {
            try {
                byte[] compareBuffer=new byte[15];
                EnvRespHdlr.findAndCompareValue((byte)0x0D, compareBuffer, (short)0) ;
            }
            catch (ArrayIndexOutOfBoundsException e) {
                bRes = true ;
            }
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 6 : Unavailable element
        testCaseNb = (byte) 6 ;
        bRes = false ;
        

        // Initialise handler
        EnvRespHdlr.clear();
        //built the first tlv
        EnvRespHdlr.appendTLV((byte)0x02,TLVBuffer,(short) 0x00, (short) 2);

        // Select a TLV
        EnvRespHdlr.findTLV(TAG_DEVICE_IDENTITIES, (byte)1) ;

        // Search a wrong TLV
        try {
            EnvRespHdlr.findAndCompareValue((byte)0x03, TLVBuffer, (short)0) ;
            
        } catch (ToolkitException e) {
            bRes = (e.getReason() == ToolkitException.UNAVAILABLE_ELEMENT) ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

        // --------------------------------------------
        // Test Case 7 : verify the current TLV
        testCaseNb = (byte) 7 ;
        bRes = false ;

        // Verify there is no selected TLV
        try {
            EnvRespHdlr.getValueLength() ;
        }
        catch (ToolkitException e) {
            bRes = (e.getReason() == ToolkitException.UNAVAILABLE_ELEMENT) ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 8 : Successful call
        testCaseNb = (byte) 8 ;
        bRes = false ;

        try {
            // Initialise handler
            EnvRespHdlr.clear();
            //built the first tlv
            EnvRespHdlr.appendTLV((byte)0x0D,TLVBuffer,(short) 0x00, (short) 17);

            if(EnvRespHdlr.findAndCompareValue((byte)0x0D, TLVBuffer, (short)0)==0)
                bRes=true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 9 : Verify current TLV
        testCaseNb = (byte) 9 ;
        bRes = false ;

        try {
            if(EnvRespHdlr.getValueLength()==(short)17)
                bRes=true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 10 : Successful call
        testCaseNb = (byte) 10 ;
        bRes = false ;

        // Initialise compare buffer
        TLVBuffer[16] = (byte)0x10;

        try {
            if(EnvRespHdlr.findAndCompareValue((byte)0x0D, TLVBuffer, (short)0)==-1)
                bRes=true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 11 : Successful call
        testCaseNb = (byte) 11 ;
        bRes = false ;

        // Initialise compare buffer
        TLVBuffer[16] = (byte)0x0F;
        TLVBuffer[0] = (byte)0x03;

        try {
            if(EnvRespHdlr.findAndCompareValue((byte)0x0D, TLVBuffer, (short)0)==1)
                bRes=true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 12 : Successful call
        testCaseNb = (byte) 12 ;
        bRes = false ;

        // Initialise compare buffer
        Util.arrayFillNonAtomic(TLVBuffer,(short)0,(short)TLVBuffer.length,(byte)0x55);
        for(short i=3;i<20;i++)
                TLVBuffer[i]=(byte)(i-3);
            TLVBuffer[2]=(byte)4;

        try {
            if(EnvRespHdlr.findAndCompareValue((byte)0x0D, TLVBuffer, (short)2)==0)
                bRes=true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 13 : Successful call with 2 Text String TLV
        testCaseNb = (byte) 13 ;
        bRes = false ;

        // Initialise compare buffer
        EnvRespHdlr.appendTLV((byte)0x0D,TLVBuffer,(short) 0x00, (short) 17);


        try {

            // Initialise compare buffer
            TLVBuffer[0] = (byte) 0x00 ;
            TLVBuffer[1] = (byte) 0x11 ;
            TLVBuffer[2] = (byte) 0x22 ;
            TLVBuffer[3] = (byte) 0x33 ;
            TLVBuffer[4] = (byte) 0x44 ;
            TLVBuffer[5] = (byte) 0x55 ;
            EnvRespHdlr.appendTLV((byte)0x0D, TLVBuffer, (short)0, (short)6) ;

            // Initialise compare buffer
            Util.arrayFillNonAtomic(TLVBuffer,(short)0,(short)TLVBuffer.length,(byte)0x55);
            for(short i=3;i<20;i++)
                TLVBuffer[i]=(byte)(i-3);
            TLVBuffer[2]=(byte)4;

            if(EnvRespHdlr.findAndCompareValue((byte)0x0D, TLVBuffer, (short)2)==0)
                bRes = true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 14 : Successful call with 2 Text String TLV
        testCaseNb = (byte) 14 ;
        bRes = false ;

        // Initialise compare buffer
        TLVBuffer[3] = (byte)1;

        try {
            if(EnvRespHdlr.findAndCompareValue((byte)0x0D, TLVBuffer, (short)2)==(short)-1)
                bRes = true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 15 : Successful call with 2 Text String TLV
        testCaseNb = (byte) 15 ;
        bRes = false ;

        // Initialise compare buffer
        TLVBuffer[3] = (byte)0;
        TLVBuffer[17] = (byte)0x0D;
        TLVBuffer[18] = (byte)0x10;

        try {
            if(EnvRespHdlr.findAndCompareValue((byte)0x0D, TLVBuffer, (short)2)==1)
                bRes=true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

        // --------------------------------------------
        // Test Case 16 : Successful call : search tag 8Dh
        testCaseNb = (byte) 16 ;
        bRes = false ;

        TLVBuffer=new byte[17];
        // Initialise compare buffer
        for(short i=0;i<17;i++)
            TLVBuffer[i]=(byte)(i);
        TLVBuffer[0]=(byte)4;


        try {

            EnvRespHdlr.clear();
            //add the first tag
            EnvRespHdlr.appendTLV((byte)0x0D, TLVBuffer , (short)0, (short)17) ;

            // Search tag 8Dh
            if(EnvRespHdlr.findAndCompareValue((byte)0x8D, TLVBuffer, (short)0)==0)
                bRes=true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 17 : Successful call : search tag 8Fh
        testCaseNb = (byte) 17 ;
        bRes = false ;

        try {
            //add the first tag
            EnvRespHdlr.appendTLV((byte)0x0F, TLVBuffer , (short)1, (short)16) ;

            //initilase the compare buffer
            for(short i=1;i<17;i++)
                TLVBuffer[(short)(i-1)]=TLVBuffer[i];

            // Search tag 8Fh
            if(EnvRespHdlr.findAndCompareValue((byte)0x8F, TLVBuffer, (short)0)==0)
                bRes=true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

        // --------------------------------------------
        // Test Case 18 : ToolkitException HANDLER_NOT_AVAILABLE
        testCaseNb = (byte) 18 ;
        bRes = false ;

        try {
            EnvRespHdlr.clear();
            EnvRespHdlr.post((byte)SW1_RP_ACK);
            try {
                EnvRespHdlr.findAndCompareValue((byte)0, dstBuffer, (short)0);
            }
            catch (ToolkitException e) {
                if (e.getReason() == ToolkitException.HANDLER_NOT_AVAILABLE) {
                    bRes = true;
                } else {
                    bRes = false;
                }
            }
        }
        catch (Exception e) {
            bRes = false;
        }
        reportTestOutcome(testCaseNb, bRes) ;

    }
}
