//-----------------------------------------------------------------------------
//    API_2_ERH_FACYB_BS_1.java
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//    Package Definition
//-----------------------------------------------------------------------------
package sim.test.toolkit.api_2_erh_facyb_bs;

//-----------------------------------------------------------------------------
//    Imports
//-----------------------------------------------------------------------------
import sim.toolkit.* ;
import javacard.framework.* ;
import sim.test.util.* ;

/**
 * Test Area : sim.toolkit.EnvelopeResponseHandler.findAndCopyValue(byte, byte[], short) method
 * 
 * Applet is triggered on Install (Install), EVENT_FORMATTED_SMS_PP_ENV
 *
 * @version 1.0.0 - 01/08/01
 * @author 3GPP T3 SWG API
 *
 * @see sim.toolkit.EnvelopeResponseHandler
 */
public class API_2_ERH_FACYB_BS_1 extends TestToolkitApplet {

    /** Registry entry concerning applet instance */
    private ToolkitRegistry oReg;

    /**
     * Constructor of the applet
     */
    public API_2_ERH_FACYB_BS_1() {
        // Specific registration to Unrecognized Envelope
        obReg.setEvent(EVENT_UNRECOGNIZED_ENVELOPE);
    }


    /**
     * Method called by the JCRE at the installation of the applet
     */
    public static void install(byte bArray[], short bOffset, byte bLength) {
        // Create a new applet instance
        API_2_ERH_FACYB_BS_1 thisApplet = new API_2_ERH_FACYB_BS_1();

        // Register the new applet instance to the JCRE
        thisApplet.register(bArray, (short)(bOffset+1), bArray[bOffset]);

        // Initialise the data of the test applet
        thisApplet.init();
    }


    /**
     * Method called by the SIM Toolkit Framework
     */
    public void processToolkit(byte event) {
        // Result of tests
        boolean bRes;

        // Number of tests
        byte testCaseNb = (byte) 0x00 ;

        // Get the system instance of the EnvelopeResponseHandler class
        EnvelopeResponseHandler EnvRespHdlr = EnvelopeResponseHandler.getTheHandler() ;

        // --------------------------------------------
        // Test Case 1 : Null as dstBuffer
        testCaseNb = (byte) 1 ;
        bRes = false ;

        byte[] TLVBuffer=new byte[20];
        byte[] dstBuffer=new byte[20];

        try {
            //initialize the buffer
            for(short i=1;i<17;i++)
                TLVBuffer[i]=(byte)(i-1);
            TLVBuffer[0]=(byte)4;

            //built the first TLV
            EnvRespHdlr.appendTLV((byte)0x0D,TLVBuffer,(short) 0x00, (short) 16);

            try {
                EnvRespHdlr.findAndCopyValue((byte)0x0D, null, (short)0) ;
            }
            catch (NullPointerException e) {
                bRes = true ;
            }
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

        // --------------------------------------------
        // Test Case 2 : dstOffset >= dstBuffer.length
        testCaseNb = (byte) 2 ;
        bRes = false ;

        try {
            try {
                EnvRespHdlr.findAndCopyValue((byte)0x0D, dstBuffer, (short)20) ;
            }
            catch (ArrayIndexOutOfBoundsException e) {
                bRes = true ;
            }
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 3 : dstOffset < 0
        testCaseNb = (byte) 3 ;
        bRes = false ;

        try {
            try {
                EnvRespHdlr.findAndCopyValue((byte)0x0D, dstBuffer, (short)-1) ;
            }
            catch (ArrayIndexOutOfBoundsException e) {
                bRes = true ;
            }
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 4 : dstOffset + length > dstBuffer.length
        testCaseNb = (byte) 4 ;
        bRes = false ;

        try {
            try {
                EnvRespHdlr.findAndCopyValue((byte)0x0D, dstBuffer, (short)5) ;
            }
            catch (ArrayIndexOutOfBoundsException e) {
                bRes = true ;
            }
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 5 : length > dstBufferLength
        testCaseNb = (byte) 5 ;
        bRes = false ;

        dstBuffer=new byte[15];

        try {
            try {
                EnvRespHdlr.findAndCopyValue((byte)0x0D, dstBuffer, (short)0) ;
            }
            catch (ArrayIndexOutOfBoundsException e) {
                bRes = true ;
            }
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 6 : Unavailable element
        testCaseNb = (byte) 6 ;
        bRes = false ;

        TLVBuffer[0]=(byte)11;
        TLVBuffer[1]=(byte)22;

        EnvRespHdlr.clear();
        //built the first tlv
        EnvRespHdlr.appendTLV((byte)0x02,TLVBuffer,(short) 0x00, (short) 2);

        // Select a TLV
        EnvRespHdlr.findTLV(TAG_DEVICE_IDENTITIES, (byte)1) ;

        try {
            EnvRespHdlr.findAndCopyValue((byte)0x03, dstBuffer, (short)0) ;
        }
        catch (ToolkitException e) {
            bRes = (e.getReason() == ToolkitException.UNAVAILABLE_ELEMENT) ;
        }
        reportTestOutcome(testCaseNb, bRes) ;



        testCaseNb = (byte) 7 ;
        bRes = false ;
        // Verify there is no current TLV
        try {
            EnvRespHdlr.getValueLength() ;
        }
        catch (ToolkitException e) {
            bRes = (e.getReason() == ToolkitException.UNAVAILABLE_ELEMENT) ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 7 : Successful call
        testCaseNb = (byte) 8 ;
        bRes = false ;
        dstBuffer=new byte[20];
        try {
            //initialize the buffer
            for(short i=1;i<17;i++)
                TLVBuffer[i]=(byte)(i-1);
            TLVBuffer[0]=(byte)4;

            //built the second TLV
            EnvRespHdlr.appendTLV((byte)0x0D,TLVBuffer,(short) 0x00, (short) 17);

            if(EnvRespHdlr.findAndCopyValue((byte)0x0D, dstBuffer, (short)0)==(short)17)
                bRes = true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 8 : Compare buffer
        testCaseNb = (byte) 9 ;
        bRes = false ;
        try {
            if(Util.arrayCompare(TLVBuffer, (short)0,dstBuffer, (short)0, (short)17)==0)
                bRes=true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

        // --------------------------------------------
        // Test Case 9 : Successful call
        testCaseNb = (byte) 10 ;
        bRes = false ;

        try {
            // Initialise dstBuffer
            Util.arrayFillNonAtomic(dstBuffer,(short)0,(short)dstBuffer.length,(byte)0x55);
                   
            if(EnvRespHdlr.findAndCopyValue((byte)0x0D, dstBuffer, (short)2)==(short)19)
                bRes=true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 10 : Compare buffer
        testCaseNb = (byte) 11 ;
        bRes = false ;

        // Initialise compareBuffer
        Util.arrayFillNonAtomic(TLVBuffer,(short)0,(short)TLVBuffer.length,(byte)0x55);
        TLVBuffer[2] = (byte)4;
        for (short i=3; i<19; i++) 
            TLVBuffer[i] = (byte)(i-3);

        try {
            if(Util.arrayCompare(TLVBuffer, (short)0,dstBuffer, (short)0, (short)20)==(short)0)
                bRes=true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

        // --------------------------------------------
        // Test Case 11 : Successful call with 2 Text String TLV
        testCaseNb = (byte) 12 ;
        bRes = false ;

        try {
            EnvRespHdlr.clear();

            TLVBuffer[2] = (byte)4;
            for (short i=3; i<19; i++)
                TLVBuffer[i] = (byte)(i-3);

            //built the first TLV
            EnvRespHdlr.appendTLV((byte)0x0D,TLVBuffer,(short) 0x00, (short) 17);
        
            //built the second TLV
            EnvRespHdlr.appendTLV((byte)0x0D,(byte)0x11);

            if(EnvRespHdlr.findAndCopyValue((byte)0x0D, dstBuffer, (short)0)==(short)17)
                bRes=true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 12 : Compare buffer
        testCaseNb = (byte) 13 ;
        bRes = false ;

        try {
            if(Util.arrayCompare(TLVBuffer, (short)0,dstBuffer, (short)0, (short)17)==0)
                bRes=true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 13 : Successful call, tag = 8Dh
        testCaseNb = (byte) 14 ;
        bRes = false ;

        try {
            EnvRespHdlr.clear();

            //built the first tlv
            EnvRespHdlr.appendTLV((byte)0x0D,TLVBuffer,(short) 0x00, (short) 17);

            if(EnvRespHdlr.findAndCopyValue((byte)0x8D, dstBuffer, (short)0)==(short)17)
                bRes=true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 14 : Compare buffer
        testCaseNb = (byte) 15 ;
        bRes = false ;

        try {
            if(Util.arrayCompare(TLVBuffer, (short)0,dstBuffer, (short)0, (short)17)==0)
                bRes=true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 15 : Successful call, tag = 8Fh
        testCaseNb = (byte) 16 ;
        bRes = false ;

        try {
            // Append TLV with tag 0Fh
            EnvRespHdlr.appendTLV((byte)0x0F,TLVBuffer,(short) 0x01, (short) 16);

            // Search tag 8Fh
            if(EnvRespHdlr.findAndCopyValue((byte)0x8F, dstBuffer, (short)0)==(short)16)
                bRes=true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;


        // --------------------------------------------
        // Test Case 16 : Compare buffer (with the length = 0 and the offset = length of the destination buffer)
        testCaseNb = (byte) 17 ;
        bRes = false ;

        try {
            if(Util.arrayCompare(TLVBuffer, (short)1,dstBuffer, (short)0, (short)16)==0)
                bRes = true;
        }
        catch (Exception e) {
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

        // --------------------------------------------
        // Test Case 17 : ToolkitException HANDLER_NOT_AVAILABLE
        testCaseNb = (byte) 18 ;
        bRes = false ;

        try {
            EnvRespHdlr.clear();
            EnvRespHdlr.post((byte)SW1_RP_ACK);
            try {
                EnvRespHdlr.findAndCopyValue((byte)0x8F, dstBuffer, (short)0);
            }
            catch (ToolkitException e) {
                if (e.getReason() == ToolkitException.HANDLER_NOT_AVAILABLE) {
                    bRes = true;
                } else {
                    bRes = false;
                }
            }
        }
        catch (Exception e) {
            bRes = false;
        }
        reportTestOutcome(testCaseNb, bRes) ;
    }
}
