//-----------------------------------------------------------------------------
//    API_2_PAH_GVBYS_1.java
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//    Package Definition
//-----------------------------------------------------------------------------
package sim.test.toolkit.api_2_pah_gvbys;

//-----------------------------------------------------------------------------
//    Imports
//-----------------------------------------------------------------------------
import sim.toolkit.* ;
import javacard.framework.* ;
import sim.test.util.* ;

/**
 * Test Area : sim.toolkit.ProactiveHandler.getValueByte() method
 * 
 * Applet is triggered on Install (Install), EVENT_FORMATTED_SMS_PP_ENV
 *
 * @version 0.0.2 - 07/12/00
 * @author 3GPP T3 SWG API
 *
 * @see sim.toolkit.ProactiveHandler
 */
public class API_2_PAH_GVBYS_1 extends TestToolkitApplet
{

    /** Registry entry concerning applet instance */
    private ToolkitRegistry oReg;

    private static final byte TYPE = (byte) 0xFF ;
    private static final byte QUALIFIER = (byte) 0xFE ;
    private static final byte DST_DEVICE = (byte) 0xFD ;

    private byte[] TEXT = new byte[240] ;
    
    /**
     * Constructor of the applet
     */
    public API_2_PAH_GVBYS_1() 
    {

    }


    /**
     * Method called by the JCRE at the installation of the applet
     */
    public static void install(byte bArray[], short bOffset, byte bLength) 
    {
        // Create a new applet instance
        API_2_PAH_GVBYS_1 thisApplet = new API_2_PAH_GVBYS_1();

        // Register the new applet instance to the JCRE            
        thisApplet.register(bArray, (short)(bOffset+1), bArray[bOffset]);

        // Initialise the data of the test applet
        thisApplet.init();    
    }
    

    /**
       * Method called by the SIM Toolkit Framework
     */
    public void processToolkit(byte event) 
    {
        // Result of tests
        boolean bRes ;
        
        // Number of tests
        byte testCaseNb = (byte) 0x00 ;
        
        // Get the system instance of the ProactiveHandler class
        ProactiveHandler proHdlr = ProactiveHandler.getTheHandler() ;

        // Result of method
        short result = 0 ;

        
        // --------------------------------------------
        // Test Case 1 : Unavailable Element
        testCaseNb = (byte) 1 ;
        bRes = false ;
        
        try {
            
            // Initialise the handler
            proHdlr.init(TYPE, QUALIFIER, DST_DEVICE) ;
            
            try {
                proHdlr.getValueByte((short)0) ;
                
            } catch (ToolkitException e) {
                bRes = (e.getReason() == ToolkitException.UNAVAILABLE_ELEMENT) ;
            }
        }
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

        
        // --------------------------------------------
        // Test Case 2 : Out of Tlv Boundaries
        testCaseNb = (byte) 2 ;
        bRes = false ;
        
        try {
            
            // Search Command Details TLV
            proHdlr.findTLV((byte)0x01, (byte)1) ;

            try {
                result = proHdlr.getValueByte((short)3) ;
            } catch (ToolkitException e) {
                bRes = (e.getReason() == ToolkitException.OUT_OF_TLV_BOUNDARIES) ;
            }
        }
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

        
        // --------------------------------------------
        // Test Case 3 : Successful call
        testCaseNb = (byte) 3 ;
        bRes = false ;
        
        try {

            // Search Command Details TLV
            proHdlr.findTLV((byte)0x01, (byte)1) ;

            // Get Qualifier
            result = proHdlr.getValueByte((short)2) ;
            
            bRes = (result == QUALIFIER) ;
        }
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;

        
        // --------------------------------------------
        // Test Case 4 : Successful call
        testCaseNb = (byte) 4 ;
        bRes = false ;
        
        try {

            // Search Device Identities TLV
            proHdlr.findTLV((byte)0x02, (byte)1) ;

            // Get Source Device
            result = proHdlr.getValueByte((short)0) ;
            
            bRes = (result == (byte)0x81) ;
        }
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;
        
        
        
        // Initialise Text buffer
        for (short i=0; i<(short)TEXT.length; i++) {
            TEXT[i] = (byte) i ;
        }
        
        // --------------------------------------------
        // Test Case 5 : Successful call
        testCaseNb = (byte) 5 ;
        bRes = false ;
        
        try {

            // Initialise handler
            proHdlr.initDisplayText((byte)0, DCS_8_BIT_DATA, TEXT, (short)0, (short)0x7E) ;
            
            // Find Text String TLV
            proHdlr.findTLV((byte)0x0D, (byte)1) ;

            result = proHdlr.getValueByte((short)0x7E) ;
            
            bRes = (result == (byte)0x7D) ;
        }
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;
        
        
        // --------------------------------------------
        // Test Case 6 : Successful call
        testCaseNb = (byte) 6 ;
        bRes = false ;
        
        try {

            // Initialise handler
            proHdlr.initDisplayText((byte)0, DCS_8_BIT_DATA, TEXT, (short)0, (short)0x7F) ;
            
            // Find Text String TLV
            proHdlr.findTLV((byte)0x0D, (byte)1) ;

            result = proHdlr.getValueByte((short)0x7E) ;
            
            bRes = (result == (byte)0x7D) ;
        }
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;
        
        
        // --------------------------------------------
        // Test Case 7 : Successful call
        testCaseNb = (byte) 7 ;
        bRes = false ;
        
        try {

            result = proHdlr.getValueByte((short)0x7F) ;
            
            bRes = (result == (byte)0x7E) ;
        }
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;
        
        
        // --------------------------------------------
        // Test Case 8 : Successful call
        testCaseNb = (byte) 8 ;
        bRes = false ;
        
        try {

            // Initialise handler
            proHdlr.initDisplayText((byte)0, DCS_8_BIT_DATA, TEXT, (short)0, (short)0xF0) ;
            
            // Find Text String TLV
            proHdlr.findTLV((byte)0x0D, (byte)1) ;

            result = proHdlr.getValueByte((short)0xF0) ;
            
            bRes = (result == (byte)0xEF) ;
        }
        catch (Exception e)    {    
            bRes = false ;
        }
        reportTestOutcome(testCaseNb, bRes) ;
        
        
        
        
        
    }
}
