//-----------------------------------------------------------------------------
//    Package Definition
//-----------------------------------------------------------------------------
package sim.test.util;

//-----------------------------------------------------------------------------
//    Imports
//-----------------------------------------------------------------------------
import sim.toolkit.*;
import javacard.framework.*;

/**
 * Definition of the super class of all the test applets.
 * The default test applet is triggered on a formatted SMS PP reception.
 * The result of each and every test case is stored in an array.
 * The results of the test can be obtained by selecting the test applet : 
 * the applet ID and the test cases results are then returned in the response data.
 *
 */
abstract public class TestToolkitApplet extends javacard.framework.Applet implements 
ToolkitInterface, ToolkitConstants {

    /*
     * Generic variables for all test applets
     */

    /**    obReg            ToolktRegistry object */
    protected ToolkitRegistry obReg;

    /**    
     *    baTestAppletId    byte array to store the identifier of the test applet, 
     *                    the first byte is the length of the AID. 
     */
    protected byte[]    baTestAppletId = new byte[17];

    /**    
     *    baTestsResults    byte array containing the result of each and every test case, 
     *                    the first byte is the number of test cases.
     */
    protected byte[]    baTestsResults = new byte[128];

    /**    
     *    baTransmission    byte array containing the data to me transmitted
     */
    protected byte[]    baTransmission = new byte[128+17];

    /**
     * Constructor of the applet
     */
    public TestToolkitApplet() {
        // Register to the SIM Toolkit Framework
        obReg = ToolkitRegistry.getEntry();

        try {
            // Set Formatted SMS PP as an activation event for the applet
            obReg.setEvent(EVENT_FORMATTED_SMS_PP_ENV);
        }
        catch (ToolkitException e) {
        }

    }

    /**
     * Method called by the sub classes to initialize the AID
     */
    public void init() {
        // Get the AID value 
        baTestAppletId[0] = JCSystem.getAID().getBytes(baTestAppletId, (short)1);
        Util.arrayFillNonAtomic(baTestsResults, (short)0, (short)baTestsResults.length, (byte)0x00 );
        Util.arrayFillNonAtomic(baTransmission, (short)0, (short)baTransmission.length, (byte)0x00 );
    }

    /**
     * Method called by the Toolkit Framework
     *
     * @param clientAID the AID of the calling application, should be GSM application
     * @param parameter 
     */
    public Shareable getShareableInterfaceObject ( AID clientAID, byte parameter) {
            return ((Shareable) this);
    }

    /**
     *    Method called by the test applet to report the result of each test case
     *
     * @param testCaseNumber test case number
     * @param result true if successful, false otherwise 
     */
    protected void reportTestOutcome(byte testCaseNumber, boolean testCaseResult) {
        // Update the total number of tests executed
        baTestsResults[0] = testCaseNumber;

        // Set the Test Case Result byte to 0xCC (for Card Compliant..) if successful 
        if (testCaseResult)
            baTestsResults[testCaseNumber] = (byte)0xCC;
        else
            baTestsResults[testCaseNumber] = (byte)0x00;
    }

    /**
     *    Method called by the test applet to report the success of a test case
     *
     * @param testCaseNumber test case number
     */
    protected void reportTestSuccess(byte testCaseNumber) {
        reportTestOutcome(testCaseNumber, true);
    }

    /**
     *    Method called by the test applet to report the failure of a test case
     *
     * @param testCaseNumber test case number
     */
    protected void reportTestFailure(byte testCaseNumber) {
        reportTestOutcome(testCaseNumber, false);
    }

    /**
     * Method called by the JCRE, once selected
     * This method allows to retrieve the detailed results of the previous execution
     * may be identical for all tests
     */
    public void process(APDU apdu) {
        if (selectingApplet()) {
			/* Construct and send the results of the tests */
            apdu.setOutgoing();
            apdu.setOutgoingLength((short)((short)(baTestAppletId[0] + baTestsResults[0]) + 
                                   (short)2));
            Util.arrayCopyNonAtomic(baTestAppletId, (short)0, baTransmission, (short)0, (short)(baTestAppletId[0] + 1));
            Util.arrayCopyNonAtomic(baTestsResults, (short)0, baTransmission, (short)(baTestAppletId[0] + 1), (short)(baTestsResults[0] + 1));
            apdu.sendBytesLong(baTransmission, (short)0, (short)((short)(baTestAppletId[0] + baTestsResults[0]) + (short)2));
		}
    }
}
